#include "q3la.h"

enum Q3LA_CFG_OPTIONS{
	CFG_OPEN_RESULTS,
	CFG_DONT_INCLUDE_BOTS,
	CFG_CHATLOG,
	CFG_WRITE_DATA_FILE,
	CFG_DONT_USE_FUNNAMES,
	CFG_SORT_ORDER,
	CFG_CUSTOM_CFG,
	CFG_GAME_SPLIT,
	CFG_MIN_KILLS,
	CFG_MIN_GAMES,
	CFG_MIN_PLAYTIME,
	CFG_MAX_PLAYERS,
	CFG_TOP_PLAYERS
};


struct q3laOption{
	char *str;
	int full;
	int action;
};

q3laOption CFG_OPTIONS[]={
	{"a",1,CFG_OPEN_RESULTS},
	{"b",1,CFG_DONT_INCLUDE_BOTS},
	{"c",1,CFG_CHATLOG},
	{"d",1,CFG_WRITE_DATA_FILE},
	{"f",1,CFG_DONT_USE_FUNNAMES},
	{"o",0,CFG_SORT_ORDER},
	{"cfg:",0,CFG_CUSTOM_CFG},
	{"g:",0,CFG_GAME_SPLIT},
	{"min_kills:",0,CFG_MIN_KILLS},
	{"min_games:",0,CFG_MIN_GAMES},
	{"min_playtime:",0,CFG_MIN_PLAYTIME},
	{"max_players:",0,CFG_MAX_PLAYERS},
	{"top",0,CFG_TOP_PLAYERS}
};

configdata::configdata()
{
	strcpy (filespec, "");
	strcpy (output_directory, "");
	strcpy (homepage_address, "http://www.planetquake.com/idda");
	strcpy (homepage_string, "Q3LA homepage");
	strcpy (configfile, "q3la.cfg");
	strcpy (font_size_title, "12");
	strcpy (font_size_text, "10");
	min_kills			= 0;
	min_games			= 0;
	min_playtime		= 0;
	max_players			= 0;
	top_players			= 0;
	game_split			= 50;
	limits_used			= false;
	no_bots				= false;
	chatlog				= false;
	q3ladata			= false;
	open_after			= false;
	q3la_css			= false;
	in_subdir			= false;
	no_funnames			= false;
	sortorder_players	= so_ratio;
	itemcount = 0; // TEST
}

configdata::~configdata()
{
}

void configdata::print_opts(int opt){
	int notfirst=0;
	for (unsigned int i=0;i<sizeof(CFG_OPTIONS)/sizeof(q3laOption);i++){
		if (CFG_OPTIONS[i].action==opt){
			if (notfirst)
				printf(",");
			printf("%s",CFG_OPTIONS[i].str);
			notfirst=1;
		};
	};
};

void configdata::print_opt_begin(){
#ifndef __LINUX_PLATFORM__
	printf("\n/");
#else
	printf("\n-");
#endif
};

void configdata::print_help(){
	printf ("\nUsage: q3la [OPTIONS] <logfile> [OPTIONS]\n");
	printf ("\nOptions:\n");
	print_opt_begin();print_opts(CFG_OPEN_RESULTS);
	printf ("\t\t  open results after analysis");
	print_opt_begin();print_opts(CFG_DONT_INCLUDE_BOTS);
	printf ("\t\t  don't include bots in results");
	print_opt_begin();print_opts(CFG_CHATLOG);
	printf ("\t\t  write chat log");
	print_opt_begin();print_opts(CFG_WRITE_DATA_FILE);
	printf ("\t\t  write data file (q3ladata.txt)");
	print_opt_begin();print_opts(CFG_DONT_USE_FUNNAMES);
	printf ("\t\t  don't use \"funnames\"");
	print_opt_begin();print_opts(CFG_SORT_ORDER);
	printf ("<x>\t\t  sortorder (<x> can be N,R,E,S,F,K,N,P,T - see readme)");
	print_opt_begin();print_opts(CFG_CUSTOM_CFG);
	printf ("<filespec>\t  use custom CFG - no spaces in <filespec> allowed");
	print_opt_begin();print_opts(CFG_MIN_KILLS);
	printf ("<x>\t  exclude players with less than <x> kills from player stats");
	print_opt_begin();print_opts(CFG_MIN_GAMES);
	printf ("<x>\t  exclude players with less than <x> games from player stats");
	print_opt_begin();print_opts(CFG_MIN_PLAYTIME);
	printf ("<x> exclude players with less than <x> seconds playtime");
	print_opt_begin();print_opts(CFG_MAX_PLAYERS);
	printf ("<x>  no more than <x> players in player stats");
	print_opt_begin();print_opts(CFG_GAME_SPLIT);
	printf ("<x>            no more than <x> games per html file (default: 50)");
	print_opt_begin();print_opts(CFG_TOP_PLAYERS);
	printf ("<x>           write additional top <x> players file");
}

int configdata::find_action(char *str){
	if (str==NULL)	return(-1);
	int lenstr=strlen(str);
	if (lenstr==0) return(-1);
	for (unsigned int i=0;i<sizeof(CFG_OPTIONS)/sizeof(q3laOption);i++) {
		int lencfg=strlen(CFG_OPTIONS[i].str);
#ifdef __LINUX_PLATFORM__
		if (lencfg <= lenstr && strncasecmp(str,CFG_OPTIONS[i].str,lencfg)==0) {
#else
		if (lencfg <= lenstr && strnicmp(str,CFG_OPTIONS[i].str,lencfg)==0) {
#endif
			return i;
		}
	};
	return(-1);
};

void configdata::read_options(int argc, char **argv)
{
	int i;  
	char tempstr[256];
	for (i = 1; i < argc; i++) {
		/* FIXME: buffer overflow in UNIX if strlen(argv[i]) > 256 */
		strcpy (tempstr, argv[i]);
		_strupr (tempstr);
		if ((tempstr[0] != '-') && (tempstr[0] != '/')) {
			strcpy (filespec, tempstr);
		} else {
			int action=find_action(tempstr+1);
			if (action>=0){
				switch (CFG_OPTIONS[action].action) {
				case CFG_MIN_KILLS: // min_xxx
					min_kills = atoi (strstr (tempstr, ":") + 1);
					limits_used = true;
					break;
				case CFG_MIN_GAMES:
					min_games = atoi (strstr (tempstr, ":") + 1);
					limits_used = true;
					break;
				case CFG_MIN_PLAYTIME:
					min_playtime = atoi (strstr (tempstr, ":") + 1);
					limits_used = true;
					break;
				case CFG_MAX_PLAYERS:
					max_players = atoi (strstr (tempstr, ":") + 1);
					limits_used = true;
					break;
				case CFG_TOP_PLAYERS:
					top_players = atoi (tempstr + 4);
					break;
				case CFG_GAME_SPLIT:
					game_split = atoi (strstr (tempstr, ":") + 1);
					break;
				case CFG_CUSTOM_CFG: // custom CFG
					strcpy (configfile, strstr (argv[i], ":") + 1);
					break;
				case CFG_DONT_INCLUDE_BOTS: // no_bots in results
					no_bots = true;
					break;
				case CFG_CHATLOG: // write chatlog
					chatlog = true;
					break;
				case CFG_DONT_USE_FUNNAMES: // no_funnames in result
					no_funnames = true;
					break;
				case CFG_WRITE_DATA_FILE: // write q3ladata.txt
					q3ladata = true;
					break;
				case CFG_OPEN_RESULTS: // open after analysis
					open_after = true;
					break;
				case CFG_SORT_ORDER: // sortorder, extend!!!
					
					switch (tempstr[2]) {
					case 'P':
						sortorder_players = so_playtime;
						break;
					case 'R':
						sortorder_players = so_ratio;
						break;
					case 'E':
						sortorder_players = so_eff;
						break;
					case 'S':
						sortorder_players = so_skill;
						break;
					case 'N':
						sortorder_players = so_name;
						break;
					case 'F':
						sortorder_players = so_frags;
						break;
					case 'C':
						sortorder_players = so_score;
						break;
					case 'K':
						sortorder_players = so_kills;
						break;
					case 'T':
						sortorder_players = so_team;
						break;
					default:
						break;
					}
					break;
				default:
					if (filespec[0] == 0) { // for filespecs like "/root/games.log"
						strcpy (filespec, tempstr);
					} else {
						printf("WARNING: unknown command line parameter %s\n",tempstr);
					}
					break;
				}
			}
		}
	}
};

void configdata::print_settings(){
	char *sort_orders_names[]={
		"by ration",
		"by eff",
		"by skill",
		"by kills",
		"by team",
		"by name",
		"by frags",
		"by playtime"
	};
	printf("\nCurrent settings:\n");
	printf("\tSkipping bots in log: %s\n", no_bots?"YES":"NO");
	printf("\tWrite chat log: %s\n", chatlog?"YES":"NO");
	printf("\tFunnames: %s\n", no_funnames?"NO":"YES");
	printf("\tMax players: %i\n",limits_used?max_players:0);
	printf("\tTop x players: %i\n", top_players);
	printf("\tMin kills: %i\n",limits_used?min_kills:0);
	printf("\tMin games: %i\n",limits_used?min_games:0);
	printf("\tMin playing time: %i\n",limits_used?min_playtime:0);
	printf("\tWriting data file: %s\n", q3ladata?"YES":"NO");	
	printf("\tSort order: %s\n",sort_orders_names[sortorder_players]);
	printf("\n");
};

void configdata::read_cfg(char* cfg_str)
{
	FILE* configfile = fopen (cfg_str, "r");
	if (configfile == 0) {
		printf ("\nNo CFG found, using default values\n");
		return;
	} 
	char line[256];
	for (;;) {
		if (fgets(line, 255, configfile) == 0) {
			if (ferror(configfile)) {
				error::msg_and_exit ("analyzedemo::parse - configfile read error");
			}
			return;
		}
		parse_cfg_line (line);
	}
}

void configdata::parse_cfg_line(char *line)
{

	// ignore comment lines
	if (line[0] == ';') {
		return;
	}

	// output directory
	get_option (line, "output_directory", output_directory); 

	// homepage address
	get_option (line, "homepage_address", homepage_address);

	// homepage string
	get_option (line, "homepage_string", homepage_string);

	// font size title
	get_option (line, "font_size_title", font_size_title);

	// font size text
	get_option (line, "font_size_text", font_size_text);

}

void configdata::get_option(char* line, char* search_str, char* cfg_str) 
{
	char* pos;
	char str[256];
	strcpy (str, search_str);
	pos = strstr (line, str) + strlen (str);
	if (pos > line) {
		get_word (pos, cfg_str);
		printf ("\n- %-20s \"%s\"", search_str, cfg_str);
	}
}

void configdata::get_word(char* pos, char *str)
{
	char* first = pos;
	while ((*first != '\r') && (*first != '\n') && *first != '\"') first++;
	char* second = first+1;
	while ((*second != '\r') && (*second != '\n') && *second != '\"') second++;
	strncpy (str, first+1, second-first-1);
	str[second-first-1] = 0;
}

void configdata::item_list() // TEST
{
	char temp[256];
	for (int a = 0; a < itemcount; a++) {
		for (int b = 0; b < itemcount; b++) {
			if (strcmp (items[a], items[b]) < 0) {
				strcpy (temp, items[a]);
				strcpy (items[a], items[b]);
				strcpy (items[b], temp);
			}
		}
	}
	for (int i = 0; i < itemcount; i++) {
		printf ("\n%s", items[i]);
	}
}
